
{-# LANGUAGE BangPatterns #-}
module Bond where

import Control.DeepSeq ( NFData(..) );
------------------------------------
-- Bond related types and functions
------------------------------------

data Bond = Bond !Parval !Coupon !TimeToMat deriving (Eq, Ord, Show)
data CashTime = CashTime !Cash !Double deriving (Eq, Ord, Show)

instance NFData CashTime where
    rnf (CashTime c t) = rnf c `seq` rnf t

instance NFData Bond where
    rnf (Bond p c t) = rnf p `seq` c `seq` rnf t

type Parval = Double
type Coupon = Double
type TimeToMat = Double
type Cash = Double
type Yield = Double

-- appCoup takes a bond and returns the bond's semi-annual interest income 
appCoup :: Bond -> Double
appCoup (Bond p c _) = (c/100)*p

-- getCashFlows takes a bond and returns a list of semi-annual interest payments for the bond
getCashFlows :: Bond -> [CashTime]
getCashFlows b@(Bond p _ m) =
    let nt = round $ m*2 :: Integer 
        coup = appCoup b/2 in 
            map (\i -> let c = if i==nt then p + coup else coup in CashTime c (fromIntegral i*0.5)) [1..nt]


-- yieldToPrice takes a bond and a yield, and returns the present value of the bond
yieldToPrice :: Bond -> Yield -> Cash
yieldToPrice b y = presValSemiCashs y $ getCashFlows b

-- presValSemiCashs takes a bond's yield and list of cash flows (also known as a list of the bond's semi-annual interest payments), and returns the present value of the bond's stream of payments
presValSemiCashs :: Yield -> [CashTime] -> Cash
presValSemiCashs y cf = sum $ map (presentValSemi y) cf

-- presentValSemi takes a bond's yield and a single semi-annual interest payment, and returns the bond's present value at that point in time
presentValSemi :: Yield -> CashTime -> Cash
presentValSemi y (CashTime c t) = c/((1 + (y/200))**(2*t))

--parseBondData turns data read in from a file into a tuple with a bond and a test yield
parseBondData :: (Double, Double, Double, Double) -> (Bond, Yield)
parseBondData (p, c, m, y) = (Bond p c m, y)
