{-# LANGUAGE BangPatterns #-}

module SeqBis where

import Bond;


--------------------------------------------
-- Functions for the bisection algorithm 
--------------------------------------------

-- bracket checks whether or not the product of f(b)*f(t) has opposite signs
bracket :: (Ord a, Num a) => t -> t -> (t -> a) -> Bool
bracket b t f = ((f b <= 0) && (f t >= 0)) || ((f b >= 0) && (f t <= 0))

-- converged checks whether the two intervals have converged or not
converged :: (Ord a, Num a) => t -> t -> (t -> a) -> a -> Bool
converged b t f err = bracket b t f && (abs(f t - f b) <= err)

-- rootSolveBracket executes the bisection algorithm to approximate the root of the input function "f" with an error value equal to the input error "err"
rootSolveBracket :: (Ord a, Num a, Fractional t) => t -> t -> (t -> a) -> a -> t
rootSolveBracket bot top f err = 
    go bot top 
    where go b t =
           let m = (b + t)/2
           in if converged b t f err then m
              else if bracket b m f then go b m
                                       else go m t

------------------------------------------
-- Testing with bond data from input file
------------------------------------------

-- compareYields compares a test yield and the calculated yield for a bond by generating its squared error
compareYields :: (Bond, Yield) -> Double
compareYields (b, yog) = calcErr (b, p, yog)
  where p = yieldToPrice b yog

-- calcErr returns the squared error for a bond's calcualted YTM and a test yield
calcErr :: (Bond, Cash, Yield) -> Double
calcErr (b, c, y) = (y - priceToYield (b,c))**2.0 

-- priceToYield takes a tuple with a bond and a price and returns the bond's yield to maturity
priceToYield :: (Bond, Cash) -> Yield
priceToYield (b, p) =
    let f y = yieldToPrice b y - p
    in rootSolveBracket (-100) 100 f 0.000001

-----------------------------------------------------------------------------------
-- Sequential run of the program 
-----------------------------------------------------------------------------------
runSeq :: [(Bond, Yield)] -> [Double]
runSeq bondTups = map compareYields bondTups

